@@ -56,6 +56,8 @@ tinymce.addI18n('lv',{
 "OR": "VAI",
 "Drop an image here": "Ievelciet att\u0113lu \u0161eit",
 "Upload": "Aug\u0161upiel\u0101d\u0113t",
+"Block": "Bloks",
+"Align": "L\u012bdzin\u0101t",
 "Default": "Parastais",
 "Circle": "Aplis",
 "Disc": "Disks",
@@ -75,6 +77,7 @@ tinymce.addI18n('lv',{
 "Source code": "Pirmkods",
 "Insert\/Edit code sample": "Ievad\u012bt\/Labot koda paraugu",
 "Language": "Valoda",
+"Code sample": "Koda paraugs",
 "Color": "Kr\u0101sa",
 "R": "R",
 "G": "G",
@@ -105,6 +108,8 @@ tinymce.addI18n('lv',{
 "Premium plugins:": "\u012apa\u0161ie spraud\u0146i:",
 "Learn more...": "Uzzin\u0101t vair\u0101k...",
 "You are using {0}": "J\u016bs lietojiet {0}",
+"Plugins": "Spraud\u0146i",
+"Handy Shortcuts": "Paroc\u012bgi \u012bsce\u013ci",
 "Horizontal line": "Horizont\u0101l\u0101 l\u012bnija",
 "Insert\/edit image": "Ievietot\/labot att\u0113lu",
 "Image description": "Apraksts",
@@ -242,6 +247,7 @@ tinymce.addI18n('lv',{
 "Show blocks": "R\u0101d\u012bt blokus",
 "Show invisible characters": "R\u0101d\u012bt neredzam\u0101s rakstz\u012bmes",
 "Words: {0}": "V\u0101rdi: {0}",
+"{0} words": "{0} v\u0101rdi",
 "File": "Datne",
 "Edit": "Labot",
 "Insert": "Ievietot",
@@ -249,5 +255,6 @@ tinymce.addI18n('lv',{
 "Format": "Format\u0113t",
 "Table": "Tabula",
 "Tools": "R\u012bki",
+"Powered by {0}": "Darb\u012bbu nodro\u0161ina {0}",
 "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Satura redaktors. Nospiediet ALT-F9 lai par\u0101d\u012btu izv\u0113lni, ALT-F10 - r\u012bkjoslu vai ALT-0 - pal\u012bdz\u012bbu."
 });
\ No newline at end of file
